/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib/gi18n.h>
#include <math.h>
#include "edecor.h"
#include "calibration.h"
#include "enpathia.h"
#include "misc.h"
#include "profile.h"
#include "protocol.h"
#include "unlock.h"

#define THRESHOLD_BOTTOM	20
#define THRESHOLD_TOP		100

gboolean unlocking = FALSE;

static struct
{
	GtkWidget*	window_unlock;
	GtkWidget*	vbox_unlock;
	GtkWidget*	label_title;
	GtkWidget*	label_message;
}
widgets;

static guint	a_previous, b_previous, c_previous;
static gint		bar_level;
static gint 	bar_speed;
static guint	stage;
static guint	n_movement_notifications;
static guint	n_stopped_notifications;

static void
check_calibration_notifications	(void);


	void
load_unlock_window (void)
{
	GtkBuilder*	builder;
	gchar*		data_file;

	/* Load the interface file */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/unlock.glade"); 

	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the unlock interface");

	g_free (data_file);

	/* Get the widgets */
#define GET_WIDGET(widget,name) \
	widgets.widget = GTK_WIDGET (gtk_builder_get_object (builder, name)); \
	if (widgets.widget == NULL) g_warning (_("Widget not found: %s"), name);

	GET_WIDGET (window_unlock, "window_unlock");
	GET_WIDGET (vbox_unlock, "vbox_unlock");
	GET_WIDGET (label_title, "label_title");
	GET_WIDGET (label_message, "label_message");

#undef GET_WIDGET

	gtk_builder_connect_signals (builder, &widgets);
	g_object_unref (G_OBJECT (builder));

	gtk_window_set_keep_above (GTK_WINDOW (widgets.window_unlock), TRUE);
	hide_pointer (widgets.window_unlock);

	edecor_decorate_window (widgets.window_unlock, widgets.vbox_unlock);
}


	void
unlock (void)
{
	if (unlocking)
		return;

	/* Change to raw mode */
	change_mode (MODE_RAW);

	bar_level = 0;
	bar_speed = 0;
	stage = 0;

	/* Show the window */
	gtk_window_present (GTK_WINDOW (widgets.window_unlock));

	update_unlock_window ();

	unlocking = TRUE;
	center_pointer ();

#ifdef __MACH__
	gtk_window_set_keep_above (GTK_WINDOW (widgets.window_unlock), TRUE);		
#endif
}

	void
stop_unlock (void)
{
	unlocking = FALSE;

	gtk_widget_hide (widgets.window_unlock);
}

	void
update_unlock_window (void)
{
	gchar* text;

	if (!sensor_connected)
		bar_level = 0;

	gtk_widget_queue_draw (widgets.window_unlock);

	if (sensor_connected)
	{
		text = g_strdup_printf ("<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>",
				_("Make gentle movements until the bar is full to start working."));
	}
	else
	{
		text = g_strdup_printf ("<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>",
				_("Sensor not detected. Please, check the connections."));
	}

	gtk_label_set_markup (GTK_LABEL (widgets.label_message), text);
	g_free (text);
}

	void
unlock_handle_raw_sample (guint a, guint b, guint c)
{
	gint	var_a, var_b, var_c;
	glong	modulus;
	gchar*	text;

	if (!unlocking || stage != 0)
		return;

	/* Make the bar grow or shrink */
	bar_level += bar_speed;

	if (bar_level < 0)
		bar_level = 0;
	else if (bar_level > 100)
		bar_level = 100;

	/* Calculate the difference with the previous sample */
	var_a = (int) a - (int) a_previous;
	var_b = (int) b - (int) b_previous;
	var_c = (int) c - (int) c_previous;

	modulus = sqrt ((var_a * var_a) + (var_b * var_b) + (var_c * var_c));

	/* If the difference is between the thresholds make the bar grow */
	if (modulus > THRESHOLD_BOTTOM && modulus < THRESHOLD_TOP)
	{
		if (bar_speed < 2)
			bar_speed++;

		/* When it reaches the end... */
		if (bar_level >= 100)
		{
			bar_level = 0;

			/* Send the configuration */
			send_cfg_speed ();
			send_cfg_time ();
			send_cfg_switches ();
			send_cfg_sensitivity ();
			send_cfg_calibration ();

			if (current_profile.config.auto_calibrate)
			{
				/* Go check the calibration */
				n_movement_notifications = 0;
				n_stopped_notifications = 0;

				stage = 1;

				text = g_strdup_printf ("<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>",
						_("Stay in the rest position, checking the calibration..."));
				gtk_label_set_markup (GTK_LABEL (widgets.label_message), text);
				g_free (text);

				change_mode (MODE_NOTIFY);
			}
			else
			{
				/* Close and start moving */
				stop_unlock ();
				change_mode (MODE_MOVEMENT);
			}
		}
	}
	/* Otherwise, make it shrink */
	else
	{
		if (bar_speed > -1)
			bar_speed--;
	}

	a_previous = a;
	b_previous = b;
	c_previous = c;

	gtk_widget_queue_draw (widgets.window_unlock);
}

	void
unlock_handle_movement (gchar movement)
{
	++n_movement_notifications;
	check_calibration_notifications ();
}

	void
unlock_handle_stopped (void)
{
	++n_stopped_notifications;
	check_calibration_notifications ();
}

	static void
check_calibration_notifications (void)
{
	if (!unlocking || stage != 1)
		return;

	/* When there are enough notifications check how many were for
	 * movement and how many for stopped */
	if (n_movement_notifications + n_stopped_notifications > 2)
	{
		stop_unlock ();

		/* If it moved too much, calibrate */
		if (n_stopped_notifications < 2)
			calibrate ();
		/* Otherwise, close and start moving */
		else
			change_mode (MODE_MOVEMENT);
	}
}

	G_MODULE_EXPORT gboolean
drawingarea_unlock_bar_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t* 	cr;
	gint		width, height;

	cr = gdk_cairo_create (widget->window);

	width	= widget->allocation.width;
	height	= widget->allocation.height,

			edecor_draw_window_background (cr, 0, 0, width, height);
	edecor_draw_progress_bar (cr, 0, 0, width, height, (gdouble) bar_level / 100.0);

	cairo_destroy (cr);

	return FALSE;
}
